from langchain_huggingface import HuggingFaceEmbeddings
from typing import Any

class HuggingFaceEmbeddings(HuggingFaceEmbeddings):
    model_version: str
    
    def __init__(self, **kwargs: Any):
        """Initialize the sentence_transformer."""
        kwargs.setdefault("model_version", "1")
        super().__init__(**kwargs)


from ...utils.gpu import get_device

def default_rag_embedder():
    """Get the default embedder for Retrieval-Augmented Generation (RAG)."""
    from ...runnables.embedders.commons import HuggingFaceEmbeddings
    model_name = "sentence-transformers/all-mpnet-base-v2"
    model_kwargs = {'device': get_device()}
    encode_kwargs = {'normalize_embeddings': False}
    return HuggingFaceEmbeddings(
        model_name=model_name,
        model_kwargs=model_kwargs,
        encode_kwargs=encode_kwargs
    )


def get_device():
    import torch
    if torch.cuda.is_available():
        device = torch.device(f"cuda:{torch.cuda.current_device()}")
    elif torch.backends.mps.is_available():
        device = torch.device("mps")
    else:
        device = torch.device("cpu")
    return device